<?php

namespace App\Http\Middleware;

use Illuminate\Http\Middleware\TrustProxies as Middleware;

class TrustProxies extends Middleware
{

    /**
     * The trusted proxies for this application.
     *
     * @var array
     */
    protected $proxies = '*';

}
